@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -142,8 +142,8 @@ static ssize_t
 stream_read (GMimeStream *stream, char *buf, size_t len)
 {
 	GMimeStreamBuffer *buffer = (GMimeStreamBuffer *) stream;
+	size_t buflen, offset;
 	ssize_t n, nread = 0;
-	size_t offset;
 	
 	if (buffer->source == NULL) {
 		errno = EBADF;
@@ -189,7 +189,8 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 		break;
 	case GMIME_STREAM_BUFFER_CACHE_READ:
 		while (len > 0) {
-			if ((n = MIN (buffer->bufend - buffer->bufptr, len)) > 0) {
+			buflen = (size_t) (buffer->bufend - buffer->bufptr);
+			if ((n = MIN (buflen, len)) > 0) {
 				memcpy (buf + nread, buffer->bufptr, n);
 				buffer->bufptr += n;
 				nread += n;
@@ -465,8 +466,8 @@ stream_seek_block_read (GMimeStream *stream, gint64 offset, GMimeSeekWhence when
 	    || (offset > 0 && offset <= buffer->buflen)) {
 		/* the position is within our pre-buffered region */
 		stream->position += offset;
-		buffer->bufptr += offset;
-		buffer->buflen -= offset;
+		buffer->bufptr += (size_t) offset;
+		buffer->buflen -= (size_t) offset;
 		
 		return stream->position;
 	}
@@ -488,7 +489,7 @@ static gint64
 stream_seek_cache_read (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 {
 	GMimeStreamBuffer *buffer = (GMimeStreamBuffer *) stream;
-	size_t len, total = 0;
+	gint64 buflen, len, total = 0;
 	gint64 pos, real;
 	ssize_t nread;
 	
@@ -537,7 +538,11 @@ stream_seek_cache_read (GMimeStream *stream, gint64 offset, GMimeSeekWhence when
 		
 		pos = buffer->bufptr - buffer->buffer;
 		
-		buffer->buflen = buffer->bufend - buffer->buffer + len;
+		buflen = (buffer->bufend - buffer->buffer) + len;
+		if (buflen < G_MAXSIZE)
+			buffer->buflen = (size_t) buflen;
+		else
+			buffer->buflen = G_MAXSIZE;
 		
 		buffer->buffer = g_realloc (buffer->buffer, buffer->buflen);
 		buffer->bufend = buffer->buffer + buffer->buflen;
@@ -770,7 +775,7 @@ g_mime_stream_buffer_gets (GMimeStream *stream, char *buf, size_t max)
 				
 				if (inptr == inend && outptr < outend) {
 					/* buffer more data */
-					unsigned int offset = buffer->bufptr - buffer->buffer;
+					size_t offset = (size_t) (buffer->bufptr - buffer->buffer);
 					
 					buffer->buflen = buffer->bufend - buffer->buffer +
 						MAX (BUFFER_GROW_SIZE, outend - outptr + 1);
@@ -806,7 +811,7 @@ g_mime_stream_buffer_gets (GMimeStream *stream, char *buf, size_t max)
 		*outptr = '\0';
 	}
 	
-	return (outptr - buf);
+	return (ssize_t) (outptr - buf);
 }
 
 
